local addon = LibStub("AceAddon-3.0"):GetAddon("Ouro Loot")

--[[ Generator:
boolean FUNC (ttype, loot, last_printed, generated, cache)
in      TTYPE:  the registered text type as passed to register_text_generator
in      LOOT:  pointer to g_loot table
in      LAST_PRINTED:  index into loot most recently formatted by this routine
in      GENERATED.TTYPE:  (string) FIFO buffer for text created by this routine;
           other parts of the GUI copy and nil out this string.  Do not change
           this string, only examine it if needed.  If the generator is called
           more than once between GUI updates, text will build up here.
in/out  GENERATED.TTYPE_pos:  if non-nil, this is the saved cursor position in
           the text window (so that it stays where the user last left it).
           Move it if you're doing something strange with the displayed text.
tmp     GENERATED.loc_TTYPE_*:  Use as needed.
out     CACHE:  Empty output table.  Accumulate generated lines here, one entry
           per visible line.  Do not terminate with a newline unless you want
           an extra blank line there.

Preconditions:
  + LAST_PRINTED < #LOOT
  + all "display-relevant" information for the main Loot tab has been filled
    out (e.g., LOOT[i].cols[3] might have extra text, etc)
  + LOOT.TTYPE is a non-nil string containing all text in the text box (and
    if the user has edited the text box, this string will be updated).  Do not
    change this, but like GENERATED.TTYPE it is available for examination.

Return true if text was created, false if nothing was done.
]]

--[[ Optional special widgets:
FUNC (ttype, editbox, container, mkbutton)
    TTYPE:  see above
    EDITBOX:  the MultiLineEditBox widget
    CONTAINER:  widget container (already has 'Regenerate' button in it)
    MKBUTTON:  function to create more widgets
]]

--local forum_warned_heroic
local function forum (_, loot, last_printed, generated, cache)
	for i = last_printed+1, #loot do
		local e = loot[i]

		if e.kind == 'loot' then
			-- assuming nobody names a toon "offspec" or "gvault"
			local disp = e.disposition or e.person
			if disp == 'offspec' then
				disp = e.person .. " " .. 'offspec'
			elseif disp == 'gvault' then
				disp = "guild vault (".. e.person .. ")"
			end
			if e.extratext_byhand then
				disp = disp .. " -- " .. e.extratext
			end
			--[[
			if e.is_heroic and (not sv_OLoot_opts.forum_use_itemid)
			   and (not forum_warned_heroic)
			then
				forum_warned_heroic = true
				addon:Print("|cffff0505WARNING:|r  Heroic items sharing the same",
					"name as normal items often display incorrectly on forums that",
					"use the item name as the identifier.  Recommend you turn on",
					"'Use item IDs' and regenerate this loot.")
			end
			cache[#cache+1] = ("[item]%s[/item]%s - %s"):format(
				(sv_OLoot_opts.forum_use_itemid and e.id or e.itemname),
				(e.count or ""),
				disp)
			]]
			local t = sv_OLoot_opts.forum_format:gsub('%$I', e.id)
			           :gsub('%$N', e.itemname)
			           :gsub('%$X', e.count or "")
			           :gsub('%$T', disp)
			cache[#cache+1] = t

		elseif e.kind == 'boss' and e.reason == 'kill' then
			-- first boss in an instance gets an instance tag, others get a blank line
			if generated.last_instance == e.instance then
				cache[#cache+1] = ""
			else
				cache[#cache+1] = "\n[b]" .. e.instance .. "[/b]"
				generated.last_instance = e.instance
			end
			cache[#cache+1] = "[i]" .. e.bosskill .. "[/i]"

		elseif e.kind == 'time' then
			cache[#cache+1] = "[b]" .. e.startday.text .. "[/b]"

		end
	end
	return true
end

local function forum_spec (_,_, container, mkbutton)
	local editbox = mkbutton("EditBox", nil, sv_OLoot_opts.forum_format,
		[[Format described in Help tab (Generated Text -> Forum Markup).  Regenerate to take effect.]])
	editbox:SetFullWidth(true)
	editbox:SetLabel("Item markup")
	editbox:SetCallback("OnEnterPressed", function(_w,event,value)
		sv_OLoot_opts.forum_format = value
	end)
	container:AddChild(editbox)
--[==[
	local w = mkbutton("CheckBox", nil, "",
		[[Use numeric IDs instead of names in [item] links (distinguishes items with same name).]])
	w:SetFullWidth(true)
	w:SetType("checkbox")
	w:SetLabel("Use item IDs")
	w:SetValue(sv_OLoot_opts.forum_use_itemid)
	w:SetCallback("OnValueChanged", function(_w,event,value)
		sv_OLoot_opts.forum_use_itemid = value
	end)
	container:AddChild(w)
]==]
end

addon:register_text_generator ("forum", [[Forum Markup]], [[BBcode ready for Ouroboros forums]], forum, forum_spec)

local function att (_, loot, last_printed, _, cache)
	for i = last_printed+1, #loot do
		local e = loot[i]

		if e.kind == 'boss' and e.reason == 'kill' then
			cache[#cache+1] = ("\n%s -- %s\n%s"):format(e.instance, e.bosskill, e.raiderlist or '<none recorded>')

		elseif e.kind == 'time' then
			cache[#cache+1] = e.startday.text

		end
	end
	return true
end

addon:register_text_generator ("attend", [[Attendance]], [[Attendance list for each kill]], att)

-- vim:noet
